/*
    ab4split.c
        Splitter program for Alien Breed IV Levelsets; divides Levelsets into
        separate files for compression
*/

// Includes
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Defines
#define DETECT_LENGTH 8
#define MAX_HEIGHT 32
#define MAX_LEVELS 8
#define MAX_WIDTH 32

// Typedefs
typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef struct
{
  BYTE  height;
  BYTE  width;
  BYTE  objectives;
  BYTE  time;
  BYTE  blockPutX;
  BYTE  blockPutY;
  BYTE  blockTakeX;
  BYTE  blockTakeY;
  BYTE  blockPutType;
  BYTE  blockTakeType;
  BYTE  type;
  BYTE  alienType;
  BYTE  finishX;
  BYTE  finishY;
  BYTE  startX;
  BYTE  startY;
  BYTE  data[MAX_HEIGHT][MAX_WIDTH];
} LEVEL;

// Main
int main(int argc, char **argv)
{
    BYTE    numLevels,
            passcodes[MAX_LEVELS - 1][6];
    WORD    hiScore;
    LEVEL   levels[MAX_LEVELS];
    FILE    *fp, *lp;
    char    *listLevelNames[] = {"level1.ab4", "level2.ab4", "level3.ab4", "level4.ab4", "level5.ab4", "level6.ab4", "level7.ab4", "level8.ab4"},
            levelsetName[13],
            detectString[DETECT_LENGTH],
            listName[13];
    int     l,
            y;

    // Check number of arguments
    if (argc != 2)
    {
        printf("Wrong number of arguments!\n");
        return -1;
    }

    // Check length of Levelset name
    if (strlen(argv[1]) != 3)
    {
        printf("Invalid Levelset name!");
        return -1;
    }

    // Interpret filename
    strcpy(levelsetName, "zab4l");
    strcat(levelsetName, argv[1]);
    strcat(levelsetName, ".ab4");

    // Open Levelset
    fp = fopen(levelsetName, "rb");
    if (fp == NULL)
    {
        printf("Error opening Levelset!\n");
        return -1;
    }

    // Load Levelset and close file
    fread(detectString, DETECT_LENGTH - 1, 1, fp);
    fread(&numLevels, 1, 1, fp);
    fread(&hiScore, 2, 1, fp);
    fread(passcodes, 6, MAX_LEVELS - 1, fp);
    for (l = 0 ; l < numLevels ; l++)
    {
        fread(&levels[l], sizeof(levels[l]) - (MAX_HEIGHT * MAX_WIDTH), 1, fp);
        for (y = 0 ; y < levels[l].height ; y++)
        {
            fread(levels[l].data[y], (int)levels[l].width, 1, fp);
        }
    }
    fclose(fp);

    // Prepare to write list file for huffman.exe
    strcpy(listName, argv[1]);
    strcat(listName, ".lst");
    lp = fopen(listName, "w");

    // Write header file (not to be compressed)
    fp = fopen("header.ab4", "wb");
    strcpy(detectString, "AB40");
    strncat(detectString, strupr(argv[1]), 3);
    fwrite(detectString, DETECT_LENGTH - 1, 1, fp);
    fwrite(&numLevels, 1, 1, fp);
    fwrite(&hiScore, 2, 1, fp);
    fwrite(passcodes, 6, MAX_LEVELS - 1, fp);
    fclose(fp);

    // Write levels to separate files and build list file simultaneously
    for (l = 0 ; l < numLevels ; l++)
    {
        fprintf(lp, "%s\n", listLevelNames[l]);
        fp = fopen(listLevelNames[l], "wb");
        fwrite(&levels[l], sizeof(levels[l]) - (MAX_HEIGHT * MAX_WIDTH), 1, fp);
        for (y = 0 ; y < levels[l].height ; y++)
        {
            fwrite(levels[l].data[y], (int)levels[l].width, 1, fp);
        }
        fclose(fp);
    }

    return 0;
}
